@@ -53,14 +53,14 @@
 **    function will clear the current function sequence and free the menu
 **    handler to accept new menu selections by the user.
 **
-**    A function that has the input flag set is expected to popup a dialog
-**    widget of some type before returning. The callbacks for this input
-**    widget, after setting appropiate substition variables, is expected to
-**    call either  menu_item_abort()  or  menu_item_continue()  as
-**    appropiate, just before it returns back to the mail application
+**    A function that has the input flag set to TRUE is expected to popup a
+**    dialog widget of some type before returning. The callbacks for this
+**    input widget, after setting appropiate substition variables, is
+**    expected to call either  menu_item_abort()  or  menu_item_continue()
+**    as appropiate, just before it returns back to the mail application
 **    processing loop.  This method will allow the application to ask the
 **    user for some input or some confirmation before continuing the menu
-**    items function sequence, set in menu config file.
+**    function sequence, as set by the user in configuration file.
 **
 **  * The arguments are strings provided by the user in the config file.
 **    Each is surrounded by either double quotes `"' or single quotes `''
@@ -81,7 +81,7 @@
 **
 **    menu_item_abort()
 **       Notify the item handler not to continue the current menu_item
-**       when the routine returns.
+**       when the current routine returns.
 **
 **    menu_item_continue()
 **       Continue processing user functions after the user has returned
@@ -92,9 +92,10 @@
 **
 ** * FUTURE:-
 **      * Figure out how to add `title' elements which do not highlight
-**      * Addition of user defined Sub-Menus.
+**      * Addition of user defined Sub-Menus (what a lot of work).
 **
 ** # Example application user menu configuration `RC' file.
+** # This may not be valid for the current application.
 ** #
 ** # Any line starting with a `#' is a comment line and is ignored
 ** # Any lines ending with a `\' as the very last character of a 
@@ -117,7 +118,7 @@
 ** # function arguments are defined by the user but can have substitions
 ** # In this example  %f  is the current filename within the application
 ** #
-** item "Save As"   input("Save As", '%f') save('%i')
+** item "Save As"       input("Save As", '%f') save('%i')
 ** # Multiple functions can be called, and are executed in the order given.
 ** # Note that the arguments can use either double or single quotes.
 ** #
@@ -125,13 +126,18 @@
 ** # some user input (somehow), while this is happing the current function
 ** # sequence is put on hold the input widget either continues or aborts
 ** # the sequence.
+** #
 ** item "Load"   input("File to load", '') \
 **               load('%i')
 ** #  a line can be continued on the next line by backslashing the EOL.
 ** #
-**
 */
+/* ====================================================================== */
+/* =================  Application Specific Definitions  ================= */
+/* ====================================================================== */
+
 #include "xbmbrowser.h"
+#include <ctype.h>
 
 #define QUIT_APP    quit_browser   /* The applications quit routine */
 #define SET_WAIT    set_busywait   /* routine to set a wait cursor */
@@ -141,34 +147,37 @@
 #define SUBSTITION  '%'            /* the substitution character */
 #define MAX_ARGS    2              /* this hardcoded in usr_item_handler() */
 
-
 /* Declare the menus the users are allowed to define (though may not) */
+/* WARNING: the first menu must be the main menu which must be defined */
 static struct {
   char       *name;     /* name of this menu the user can define */
   Widget     *menu;     /* ptr to variable holding this user-menu */
 } decl_menus[] = {
+   { "main",      &menu_main      },  /* main menu -- menu button at top */
    { "global",    &menu_global    },  /* global menu -- button 2 */
-   { "bitmap",    &menu_bitmap    },  /* bitmap menu -- but 3 -- xbm files */
+   { "bitmap",    &menu_bitmap    },  /* bitmap menu -- but 3 on xbm files */
    { "pixmap",    &menu_pixmap    },  /* pixmap menu */
    { "directory", &menu_directory },  /* directory menu */
    { "other",     &menu_other     },  /* other files menu */
    { NULL, NULL }
 };
 
-/*  Declare functions the user menu items can call */
+/* Declare functions the user menu items can call */
+/* WARNING: the first function must be the quit function */
 static struct {  
   char       *name;        /* the name of function in the RC file */
-  void       (*funct)();   /* the function Pointer */
-  Cardinal   numargs;      /* number of arguments this function needs */
-  Boolean    input;        /* this function performs a input popup sequence */
+  void      (*funct)();    /* the function Pointer */
+  Cardinal    numargs;     /* number of arguments this function needs */
+  Boolean     input;       /* this function performs a input popup sequence */
 } decl_functs[] = {
-   { "scan",    scan_bitmaps,   0, False },   /* full directory scan */
-   { "rescan",  rescan_bitmaps, 0, False },   /* fast dir rescan */
-   { "chdir",   change_dir,     1, False },   /* change directory */
-   { "exec",    exec_string,    1, False },   /* external command */
-   { "confirm", user_confirm,   1, True  },   /* user confirm dialog */
-   { "input",   input_string,   2, True  },   /* user input dialog */
-   { "quit",    quit_browser,   0, False },   /* cleanup and exit program */
+   { "quit",     quit_browser,    0, False },   /* cleanup and exit program */
+   { "scan",     scan_images,     0, False },   /* full directory scan */
+   { "rescan",   rescan_images,   0, False },   /* fast dir rescan */
+   { "chdir",    change_dir,      1, False },   /* change directory */
+   { "exec",     exec_string,     1, False },   /* external command */
+   { "confirm",  user_confirm,    1, True  },   /* user confirm dialog */
+   { "input",    input_string,    2, True  },   /* user input dialog */
+   { "selected", file_selected,   0, False },   /* abort on no selection */
    { NULL, NULL, 0 }
 };
 
@@ -183,15 +192,16 @@ static struct {
    { 'd',  dir_name  },  /* %d = the current directory */
    { 'f',  file_name },  /* %f = the file selected in this directory */
    { 'b',  base_name },  /* %b = basename of the selected file */
-   { 's',  suffix   },   /* %s = suffix of the selected file (EG: ".xbm" ) */
-   { 'i',  input    },   /* %i = User input from input() function */
+   { 's',  suffix    },  /* %s = suffix of the selected file (EG: ".xbm" ) */
+   { 'i',  input     },  /* %i = User input from input() function */
+   { 'D',  init_dir  },  /* %D = initial startup directory */
    { '\0', NULL }
 };
 /* Full path of selected file  = "%d%f". Also  "%f" = "%b%s" */
 
 
 /* ====================================================================== */
-/* =============  End of application specific definitions  ============== */
+/* =============  End of Application Specific Definitions  ============== */
 /* ====================================================================== */
 
 /* --------------- Menu Handler and Argument Substition ----------------- */
@@ -226,7 +236,7 @@ menu_item_continue()
 ** one of these functions may have popped up.
 */
 {
-  char         arg[MAX_ARGS][MAX_STRING]; /* expanded argument strings */
+  char         arg[MAX_ARGS][MAX_STRING]; /* arguments to pass to user functs */
   char        *a, *d, *s;        /* string pointers arg, dest, sub-string */
   int          i, j, left, len;  /* temp variables */
 
@@ -288,13 +298,16 @@ menu_item_continue()
       *d = '\0';  /* finish of the string */
     } /* for each argument */
 
+    /* FUNCTION CALL */
     /* Make the actual function call requested by the user */
     /* HARDCODED -- MAX_ARGS is limited by this bit of code */
     (next_function->funct)(arg[0], arg[1]);
 
+    /* if the user called  menu_item_abort()  abort sequence now */
     if( next_function == NULL )
       goto menu_return;
 
+    /* do we have to wait for a popup before continuing? */
     if( next_function->input ) {
       /* Presumably the function just called popped up a input dialog of
       ** some kind, and was successful in doing so.  OK, increment the
@@ -309,14 +322,14 @@ menu_item_continue()
     /* increment to the next function (if any) */
     next_function = next_function->next;
 
-  } /* while next function is not empty */
+  } /* while next function is not an empty list */
 
   /* finished */
   goto menu_return;
 
 argument_overflow:  /* the argument after substitutions is too big! */
   fprintf(stderr, "Argument for user menu function overflows argument\n");
-  fprintf(stderr, "buffer after all substutions are made.\n" );
+  fprintf(stderr, "buffer after all substutions are made. -- ABORTING\n" );
   /* just abort this function sequence */
   menu_item_abort();
 
@@ -367,7 +380,7 @@ typedef enum {
    KEYWORD, STRING, CHAR, EOL
 } Token;
 
-static FILE   *file;                        /* given filename being read */
+static FILE   *file_ptr;                    /* given filename being read */
 static char   *configname;                  /* given filename being read */
 static char    token_string[MAX_STRING+1];  /* the actual token read */
 static int     line = 1;                    /* the current line number */
@@ -391,7 +404,7 @@ get()
 */
 { int c;
 
-  c = fgetc(file);
+  c = fgetc(file_ptr);
   if( c == EOF || c == '\n' ) {
     c = '\n';   line++;
   }
@@ -407,7 +420,7 @@ unget( c )
   char c;
 {
   if( c == '\n' ) line--;
-  ungetc( c, file );
+  ungetc( c, file_ptr );
 }
   
 
@@ -711,9 +724,7 @@ parse_item()
 void
 read_user_menus(parent, rc, name)
 /* Read the config file and setup all menus required
-** This routine assumes that two defines are available
-**   USER_RC      The Users RC file in the current users home directory
-**   DEFAULT_RC   The default RC file if  USER_RC is not found.
+** from the given file name and descriptor
 */
   Widget  parent;   /* parent widget for user menus */
   FILE   *rc;       /* file to read config from */
@@ -721,7 +732,7 @@ read_user_menus(parent, rc, name)
 {
   Token  token;     /* the current token from config file */
 
-  file       = rc;    /* save these for use by other paseing routines */
+  file_ptr   = rc;    /* save these for use by other paseing routines */
   configname = name;
 
   while( !feof(rc) ) {   /* while not end of file and no error */
@@ -762,5 +773,46 @@ read_user_menus(parent, rc, name)
          token_string, configname, line );
     QUIT_APP();
   }
+
+
+  /* If no user menu "main" was created -- create a plain quit menu */
+  if ( *(decl_menus[0].menu) == NULL ) {
+    Widget    item;
+    Functs   *quit;
+    int       j;
+
+    fprintf(stderr, "WARNING: menu \"main\" not created in \"%s\"\n",
+                       configname );
+    fprintf(stderr, "         creating default quit menu\n");
+
+    /* create default menu for "main" with a title */
+    *(decl_menus[0].menu) = XtVaCreatePopupShell(
+             decl_menus[0].name, simpleMenuWidgetClass, parent,
+             XtNlabel,  "No Main Menu Found",
+             NULL );
+    cur_menu = *(decl_menus[0].menu);
+    parse_line();
+
+    /* create a quit menu item */
+    item = XtVaCreateManagedWidget(
+           token_string, smeBSBObjectClass, cur_menu,
+           XtNlabel,    "Quit Application..",
+           XtNjustify,  XtJustifyLeft,
+           NULL );
+
+    /* which calls the quit function (and on others) */
+    quit = (Functs *) XtMalloc( sizeof( Functs ) );
+    quit->next   = NULL;
+    quit->funct  = decl_functs[0].funct;
+    quit->input  = decl_functs[0].input;
+
+    /* and which contains no arguments */
+    for( j = 0; j < MAX_ARGS; j++ )
+      quit->arg[j] = NULL;
+
+    /* assign it to the item in the menu */
+    XtAddCallback(item, XtNcallback, menu_item_handler, quit);
+  }
+
 }
 
